package SupplementalCFTags::Plugin;

use strict;
use MT;
use CustomFields::Field;

sub load_custom_fields {
    my ($ctx, $args, $cond) = @_;

    my (%terms, %args, %sys_terms);
    $ctx->set_blog_load_context($args, \%terms, \%args)
        or return $ctx->error($ctx->errstr);
    $sys_terms{blog_id} = 0;
    if ($args->{obj_type}) {
        my @obj_type = split ',', $args->{obj_type};
        $terms{obj_type} = \@obj_type;
        $sys_terms{obj_type} = \@obj_type;
    }
    if ($args->{type}) {
        my @type = split ',', $args->{type};
        $terms{type} = \@type;
        $sys_terms{type} = \@type;
    }
    my @blog_fields = CustomFields::Field->load(\%terms, \%args);
    my @sys_fields = ();
    if (!$args->{ignore_system}) {
        @sys_fields = CustomFields::Field->load(\%sys_terms);
    }
    my @fields = (@blog_fields, @sys_fields);
    my @res = ();
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    my $i = 0;
    my $vars = $ctx->{__stash}{vars} ||= {};
    my $field_count = scalar @fields;
    for my $field (@fields) {
        local $ctx->{__stash}{field} = $field;
        local $vars->{__first__} = !$i;
        local $vars->{__last__} = ($i == $field_count - 1);
        local $vars->{__odd__} = ($i % 2) == 0;
        local $vars->{__even__} = ($i % 2) == 1;
        local $vars->{__counter__} = $i + 1;
        defined(my $out = $builder->build($ctx, $tokens))
            or return $ctx->error($builder->errstr);
        push @res, $out;
        $i++;
    }
    my $glue = $args->{glue} || '';
    return join $glue, @res;
}

sub load_custom_field_info {
    my ($ctx, $args, $cond) = @_;
    my $plugin = MT->component('SupplementalCFTags');

    if (!$args->{basename}) {
        return $ctx->error($plugin->translate('Please specify basename modifier'));
    }
    my (%terms, %args);
    $ctx->set_blog_load_context($args, \%terms, \%args)
        or return $ctx->error($ctx->errstr);
    $terms{basename} = $args->{basename};
    $args{limit} = 1;
    my $field = CustomFields::Field->load(\%terms, \%args);
    if (!defined($field) && !$args->{ignore_system}) {
        $field = CustomFields::Field->load({
                     basename => $args->{basename},
                     blog_id => 0
                 });
    }
    return $ctx->error($plugin->translate('Not found field [_1].', $args->{basename}))
        if !$field;
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    local $ctx->{__stash}{field} = $field;
    defined(my $out = $builder->build($ctx, $tokens))
        or return $ctx->error($builder->errstr);
    return $out;
}

sub custom_field_options {
    my ($ctx, $args, $cond) = @_;

    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    my @res = ();
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    my @options = split ',', $field->options;
    my $option_count = scalar @options;
    my $i = 0;
    my $vars = $ctx->{__stash}{vars} ||= {};
    for my $option (@options) {
        local $ctx->{__stash}{field_option_value} = $option;
        local $vars->{__first__} = !$i;
        local $vars->{__last__} = ($i == $option_count - 1);
        local $vars->{__odd__} = ($i % 2) == 0;
        local $vars->{__even__} = ($i % 2) == 1;
        local $vars->{__counter__} = $i + 1;
        defined(my $out = $builder->build($ctx, $tokens))
            or return $ctx->error($builder->errstr);
        push @res, $out;
        $i++;
    }
    my $glue = $args->{glue} || '';
    return join $glue, @res;
}

sub custom_field_option_value {
    my ($ctx, $args, $cond) = @_;
    my $plugin = MT->component('SupplementalCFTags');

    unless(defined($ctx->stash('field_option_value'))) {
        return $ctx->error($plugin->translate('This tag must be uesd in block of MTCustomFieldOptions tag'));
    }
    return $ctx->stash('field_option_value');
}

sub custom_field_tag_name {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->tag;
}

sub custom_field_object {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->obj_type;
}

sub custom_field_type {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->type;
}

sub custom_field_blog_id {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->blog_id;
}

sub custom_field_id {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->id;
}

sub custom_field_default_value {
    my ($ctx, $args, $cond) = @_;
    my $field = $ctx->stash('field')
        or return _no_field($ctx);
    return $field->default;
}

sub _no_field {
    my ($ctx) = @_;
    return $ctx->error(MT->translate(
        "You used an '[_1]' tag outside of the context of the correct content; ",
        $ctx->stash('tag')));
}

1;
